@@ -100,20 +100,13 @@ GRP_duplicated <- function(GRP, all = FALSE){
   if (all){
     out <- (sizes > 1L)[group_id]
   } else {
-    out <- frowid(group_id, g = GRP) > 1L
+    out <- frowid(GRP) > 1L
   }
   out
 }
 # Alternative that just returns dup indices
 GRP_which_duplicated <- function(GRP, all = FALSE){
-  sizes <- GRP_group_sizes(GRP)
-  group_id <- GRP_group_id(GRP)
-  if (all){
-    out <- collapse::whichv(sizes[group_id], 1L, invert = TRUE)
-  } else {
-    out <- collapse::whichv(frowid(group_id, g = GRP), 1L, invert = TRUE)
-  }
-  out
+  cpp_which(GRP_duplicated(GRP, all))
 }
 calc_sorted_group_starts <- function(group_sizes){
   cpp_sorted_group_starts(as.integer(group_sizes))
@@ -130,8 +123,7 @@ GRP_starts <- function(GRP, use.g.names = FALSE,
     if (GRP_is_sorted(GRP)){
       out <- calc_sorted_group_starts(GRP_sizes)
       # For factors with 0 size, replace calculated group starts with 0
-      collapse::setv(out, collapse::whichv(GRP_sizes, 0L),
-                     0L, vind1 = TRUE)
+      out[cpp_which(GRP_sizes == 0L)] <- 0L
     } else {
       if (is.null(loc)){
         loc <- GRP_loc(GRP, use.g.names = FALSE)
@@ -140,15 +132,14 @@ GRP_starts <- function(GRP, use.g.names = FALSE,
       # Accounting for factors with no data
       if (collapse::anyv(GRP_sizes, 0L)){
         out <- integer(length(loc))
-        collapse::setv(out, which(GRP_sizes != 0L),
-                       gstarts, vind1 = TRUE)
+        out[cpp_which(GRP_sizes != 0L)] <- gstarts
       } else {
         out <- gstarts
       }
     }
   }
   if (is.null(out)){
-    out <- integer(0)
+    out <- integer()
   }
   if (use.g.names){
     names(out) <- GRP_names(GRP)
@@ -162,7 +153,7 @@ GRP_ends <- function(GRP, use.g.names = FALSE,
   if (GRP_is_sorted(GRP)){
     out <- calc_sorted_group_ends(GRP_sizes)
     # For factors with 0 size, replace 0 with NA
-    collapse::setv(out, collapse::whichv(GRP_sizes, 0L), 0L, vind1 = TRUE)
+    out[cpp_which(GRP_sizes == 0L)] <- 0L
   } else {
     if (is.null(loc)){
       loc <- GRP_loc(GRP, use.g.names = FALSE)
@@ -171,13 +162,13 @@ GRP_ends <- function(GRP, use.g.names = FALSE,
     # Accounting for factors with no data
     if (collapse::anyv(GRP_sizes, 0L)){
       out <- integer(length(loc))
-      collapse::setv(out, which(GRP_sizes != 0L), gends, vind1 = TRUE)
+      out[cpp_which(GRP_sizes != 0L)] <- gends
     } else {
       out <- gends
     }
   }
   if (is.null(out)){
-    out <- integer(0)
+    out <- integer()
   }
   if (use.g.names){
     names(out) <- GRP_names(GRP)
@@ -186,18 +177,45 @@ GRP_ends <- function(GRP, use.g.names = FALSE,
 }
 # Extract group order from GRP object safely
 GRP_order <- function(GRP){
+                      ### Only use the below arguments
+                      ### If GRP_order is called from radixorderv2
+                      ### Otherwise leave as is
+                      # starts = TRUE, group.sizes = FALSE, sort = TRUE){
   out <- GRP[["order"]]
   if (is.null(out)){
     group_id <- GRP_group_id(GRP)
     if (GRP_is_sorted(GRP) || is_sorted(group_id)){
       out <- seq_along(group_id)
-      attributes(out) <- list(starts = GRP_starts(GRP),
-                              maxgrpn = collapse::fmax(GRP_group_sizes(GRP)),
-                              sorted = TRUE)
+      # This should not be used unless through radixorderv
+      # if (group.sizes){
+      #   attributes(out) <- list("starts" = GRP_starts(GRP),
+      #                           "group.sizes" = GRP_group_sizes(GRP),
+      #                           "maxgrpn" = collapse::fmax(GRP_group_sizes(GRP)),
+      #                           "sorted" = TRUE)
+      # } else {
+        attributes(out) <- list(starts = GRP_starts(GRP),
+                                maxgrpn = collapse::fmax(GRP_group_sizes(GRP)),
+                                sorted = TRUE)
+      # }
     } else {
-      out <- collapse::radixorderv(group_id, starts = TRUE)
+      out <- collapse::radixorderv(group_id,
+                                   starts = TRUE,
+                                   group.sizes = FALSE,
+                                   sort = TRUE)
     }
   }
+  # else if (group.sizes){
+  #   ### Again, if we need group.sizes from radixorderv2
+  #   ### We simply add group.sizes and reorder the attributes correctly
+  #   starts <- attr(out, "starts")
+  #   maxgrpn <- attr(out, "maxgrpn")
+  #   sorted <- attr(out, "sorted")
+  #   out <- strip_attrs(out)
+  #   attributes(out) <- list("starts" = starts,
+  #                           "group.sizes" = GRP_group_sizes(GRP),
+  #                           "maxgrpn" = maxgrpn,
+  #                           "sorted" = sorted)
+  # }
   out
 }
 # Making this because of a bug when gsplit(NULL, GRP(x, sort = FALSE))
@@ -297,7 +315,7 @@ df_as_GRP <- function(data, return.groups = TRUE, return.order = TRUE){
   if (return.groups){
     if (has_factor){
       gstarts <- integer(n_groups)
-      gstarts[collapse::whichv(gsizes, 0L, invert = TRUE)] <-
+      gstarts[cpp_which(gsizes == 0L, invert = TRUE)] <-
         GRP_loc_starts(gdata[[".rows"]])
     } else {
       gstarts <- GRP_loc_starts(gdata[[".rows"]])
@@ -393,33 +411,6 @@ GRP.Interval <- function(X, ...){
 GRP.NULL <- function(X, ...){
   NULL
 }
-GRP_row_id <- function(GRP, ascending = TRUE){
-  size <- GRP_data_size(GRP)
-  has_order <- !is.null(GRP[["order"]])
-  is_sorted <- GRP_is_sorted(GRP)
-  if (has_order || is_sorted){
-    seq_sizes <- GRP_group_sizes(GRP)
-    if (ascending){
-      start <- 1L
-      every <- 1L
-    } else {
-      start <- seq_sizes
-      every <- -1L
-    }
-    out <- sequence2(seq_sizes, from = start, by = every)
-    if (!is_sorted){
-      out <- collapse::greorder(out, g = GRP)
-    }
-  } else {
-    if (!ascending){
-      o <- seq.int(from = size, to = min(1L, size), by = -1L)
-    } else {
-     o <- NULL
-    }
-    out <- grouped_seq_len(size, check.o = FALSE, o = o, g = GRP)
-  }
-  out
-}
 # Use this to turn a sorted group ID into a GRP when you have basic group information
 sorted_group_id_to_GRP <- function(x,
                                    n_groups,
@@ -531,6 +522,7 @@ greorder2 <- function(x, g, ...){
     collapse::greorder(x, g = g, ...)
   }
 }
+
 grouped_seq_len <- function(length, g = NULL, ...){
   if (is_integerable(length)){
     ones <- collapse::alloc(1L, length)
@@ -539,3 +531,77 @@ grouped_seq_len <- function(length, g = NULL, ...){
   }
   collapse::fcumsum(ones, g = g, na.rm = FALSE, ...)
 }
+
+# Fast grouped row IDs when you already have  GRP object
+GRP_row_id <- function(g, ascending = TRUE){
+  g <- GRP2(g)
+  size <- GRP_data_size(g)
+  # If groups are sorted we can use sequence()
+  if (GRP_is_sorted(g)){
+    group_sizes <- GRP_group_sizes(g)
+    if (ascending){
+      start <- 1L
+      every <- 1L
+    } else {
+      start <- group_sizes
+      every <- -1L
+    }
+    out <- sequence2(group_sizes, from = start, by = every)
+  } else {
+    if (!ascending){
+      o <- seq.int(length.out = size, from = size, by = -1L)
+      out <- collapse::fcumsum(seq_ones(size), g = g, na.rm = FALSE,
+                               o = o, check.o = FALSE)
+    } else {
+      out <- collapse::fcumsum(seq_ones(size), g = g, na.rm = FALSE)
+    }
+  }
+  out
+}
+
+radixorderv2 <- function(x, starts = FALSE, sort = TRUE, group.sizes = FALSE,
+                         ...){
+  if (is.null(x)){
+    return(NULL)
+  }
+  if (is_GRP(x)){
+    return(GRP_order(x))
+    # return(GRP_order(x, group.sizes = group.sizes))
+  }
+  if (is_interval(x)){
+    x <- interval_separate(x)
+  }
+  if (is.list(x) && list_has_interval(x)){
+    x <- mutate_intervals_to_ids(x)
+  }
+  collapse::radixorderv(x, starts = starts, sort = sort, group.sizes = group.sizes,
+                        ...)
+}
+grouped_row_id <- function(x, ascending = TRUE){
+  o <- radixorderv2(x, starts = FALSE, sort = FALSE, group.sizes = TRUE)
+  if (is.null(o)){
+    return(seq_len(vec_length(x)))
+  }
+  # Basically the order item of a GRP object
+  # Doesn't naturally come with group sizes
+  if (is_GRP(x)){
+    group_sizes <- GRP_group_sizes(x)
+  } else {
+    group_sizes <- attr(o, "group.sizes")
+  }
+  starts <- attr(o, "starts")
+  is_sorted <- isTRUE(attr(o, "sorted"))
+  if (is_sorted){
+    if (ascending){
+      start <- 1L
+      every <- 1L
+    } else {
+      start <- group_sizes
+      every <- -1L
+    }
+    out <- sequence2(group_sizes, from = start, by = every)
+  } else {
+    out <- cpp_row_id(o, group_sizes, ascending)
+  }
+  out
+}
