@@ -102,10 +102,10 @@ time_by <- function(data, time, time_by_unit = NULL,
                     from = NULL, to = NULL,
                     .name = "{.col}",
                     .add = FALSE,
-                    time_type = c("auto", "duration", "period"),
+                    time_type = getOption("timeplyr.time_type", "auto"),
                     time_floor = FALSE,
                     week_start = getOption("lubridate.week.start", 1),
-                    roll_month = "preday", roll_dst = "pre",
+                    roll_month = getOption("timeplyr.roll_month", "preday"), roll_dst = getOption("timeplyr.roll_dst", "boundary"),
                     .time_by_group = TRUE){
   check_is_df(data)
   data_nms <- names(data)
