@@ -63,7 +63,7 @@
 #' @export
 time_gaps <- function(x, time_by = NULL,
                       g = NULL, use.g.names = TRUE,
-                      time_type = c("auto", "duration", "period"),
+                      time_type = getOption("timeplyr.time_type", "auto"),
                       check_time_regular = FALSE){
   g <- GRP2(g)
   check_data_GRP_size(x, g)
@@ -85,7 +85,7 @@ time_gaps <- function(x, time_by = NULL,
   time_tbl <- fenframe(x,
                        name = "group",
                        value = "time")
-  time_not_na <- !is.na(time_tbl[["time"]])
+  time_not_na <- cpp_which(is.na(time_tbl[["time"]]), invert = TRUE)
   time_tbl <- df_row_slice(time_tbl, time_not_na)
   time_full_tbl <- fenframe(time_seq,
                             name = "group",
@@ -103,7 +103,7 @@ time_gaps <- function(x, time_by = NULL,
 time_num_gaps <- function(x, time_by = NULL,
                           g = NULL, use.g.names = TRUE,
                           na.rm = TRUE,
-                          time_type = c("auto", "duration", "period"),
+                          time_type = getOption("timeplyr.time_type", "auto"),
                           check_time_regular = FALSE){
   check_is_time_or_num(x)
   time_type <- match_time_type(time_type)
@@ -134,7 +134,7 @@ time_num_gaps <- function(x, time_by = NULL,
   out <- full_seq_size - n_unique
   if (!na.rm){
     nmiss <- fnmiss(x, g = g, use.g.names = FALSE)
-    out[nmiss > 0] <- NA
+    out[cpp_which(nmiss > 0)] <- NA
   }
   if (use.g.names){
     names(out) <- GRP_names(g)
@@ -146,7 +146,7 @@ time_num_gaps <- function(x, time_by = NULL,
 time_has_gaps <- function(x, time_by = NULL,
                           g = NULL, use.g.names = TRUE,
                           na.rm = TRUE,
-                          time_type = c("auto", "duration", "period"),
+                          time_type = getOption("timeplyr.time_type", "auto"),
                           check_time_regular = FALSE){
   time_num_gaps(x, time_by = time_by,
                 g = g, use.g.names = use.g.names,
