@@ -53,7 +53,7 @@
 #'
 #' @export
 time_id <- function(x, time_by = NULL, g = NULL, na_skip = TRUE,
-                    time_type = c("auto", "duration", "period"),
+                    time_type = getOption("timeplyr.time_type", "auto"),
                     shift = 1L){
   check_is_time_or_num(x)
   time_by <- time_by_get(x, time_by)
@@ -65,7 +65,8 @@ time_id <- function(x, time_by = NULL, g = NULL, na_skip = TRUE,
   # Make sure we don't lose precision when converting to integer
   out <- trunc(elapsed)
   elapsed_rounded <- round(elapsed)
-  elapsed_truncated <- which(double_equal(elapsed, elapsed_rounded) & (elapsed_rounded != out))
+  elapsed_truncated <- cpp_which(cppdoubles::double_equal(elapsed, elapsed_rounded) &
+                                   (elapsed_rounded != out))
   out[elapsed_truncated] <- elapsed_rounded[elapsed_truncated]
   as.integer(out) + shift
   # as.integer(elapsed + 1e-10) + shift
