@@ -85,7 +85,7 @@
 time_is_regular <- function(x, time_by = NULL,
                             g = NULL, use.g.names = TRUE,
                             na.rm = TRUE,
-                            time_type = c("auto", "duration", "period"),
+                            time_type = getOption("timeplyr.time_type", "auto"),
                             allow_gaps = TRUE,
                             allow_dups = TRUE){
   check_is_time_or_num(x)
@@ -105,14 +105,18 @@ time_is_regular <- function(x, time_by = NULL,
                            time_type = time_type, rolling = FALSE,
                            na_skip = na.rm,
                            fill = 0)
-  roll_time_diff <- fdiff2(telapsed, fill = 0, g = g)
+  if (na.rm){
+    roll_time_diff <- fdiff2(roll_na_fill(telapsed, g = g), fill = 0, g = g)
+  } else {
+    roll_time_diff <- fdiff2(telapsed, fill = 0, g = g)
+  }
   # out <- vapply(collapse::gsplit(telapsed, g = g),
   #               function(x) is_whole_number(x, na.rm = na.rm),
   #               logical(1))
   # double_equal(telapsed, round(telapsed))
   is_whole_num <- are_whole_numbers(telapsed)
   if (na.rm){
-    is_whole_num[is.na(is_whole_num)] <- TRUE
+    is_whole_num[collapse::whichNA(is_whole_num)] <- TRUE
   }
   n_whole_num <- collapse::fsum(is_whole_num, g = g, use.g.names = FALSE,
                                 na.rm = na.rm, fill = FALSE)
@@ -141,7 +145,7 @@ time_is_regular <- function(x, time_by = NULL,
 # Ungrouped version.
 # time_is_reg <- function(x, time_by = NULL,
 #                         na.rm = TRUE,
-#                         time_type = c("auto", "duration", "period"),
+#                         time_type = getOption("timeplyr.time_type", "auto"),
 #                         allow_gaps = TRUE,
 #                         allow_dups = TRUE){
 #   check_is_time_or_num(x)
